# frozen_string_literal: true
module Roqua
  module Healthy
    class Error < StandardError; end
    class ConnectionError < Error; end
    class NACK < Error; end

    class IllegalMirthResponse < ConnectionError; end
    class Timeout < ConnectionError; end
    class HostUnreachable < ConnectionError; end
    class ConnectionRefused < ConnectionError; end
    class NoRouteToHost < ConnectionError; end

    class UnknownFailure < Error; end
    class PatientIdNotInRemote < Error; end
    class PatientNotFound < Error; end
    class AuthenticationFailure < Error; end

    module MirthErrors
      class WrongPatient < Error; end
    end

    def self.convert_generic_errors_to_healthy_errors
      yield
    rescue RestClient::Exceptions::ReadTimeout, RestClient::Exceptions::OpenTimeout => error
      raise ::Roqua::Healthy::Timeout, error.message
    rescue ::Timeout::Error => error
      raise ::Roqua::Healthy::Timeout, error.message
    rescue Errno::ETIMEDOUT => error
      raise ::Roqua::Healthy::Timeout, error.message
    rescue Errno::EHOSTUNREACH => error
      raise ::Roqua::Healthy::HostUnreachable, error.message
    rescue Errno::ECONNREFUSED => error
      raise ::Roqua::Healthy::ConnectionRefused, error.message
    end

    ERRORS = {
      'Timeout waiting for ACK' =>
        ::Roqua::Healthy::Timeout,
      "Unable to connect to destination\tSocketTimeoutException\tconnect timed out" =>
        ::Roqua::Healthy::Timeout,
      'ERROR: Timeout waiting for response' =>
        ::Roqua::Healthy::Timeout,
      'ERROR: SocketTimeoutException: connect timed out' =>
        ::Roqua::Healthy::Timeout,
      "Unable to connect to destination\tConnectException\tConnection refused" =>
        ::Roqua::Healthy::ConnectionRefused,
      'ERROR: ConnectException: Connection refused' =>
        ::Roqua::Healthy::ConnectionRefused,
      'ERROR: NoRouteToHostException: No route to host' =>
        ::Roqua::Healthy::NoRouteToHost,
      "ERROR: NACK sent from receiver. (Rejected)" =>
        ::Roqua::Healthy::NACK,
      "ERROR: NACK sent from receiver. (Error)" => # ie. Patient not found
        ::Roqua::Healthy::NACK
    }.freeze
  end
end
